#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>

#ifndef NULL
	#define NULL ((void *) 0)
#endif

typedef struct _DList DList;

struct _DList {
	char *str;
	DList *next;
	DList *prev;
};


static DList *
d_list_new (const char *dirname)
{
	DList *list = NULL, *liststart = NULL;
	DIR *dir;
	struct dirent *ent;
	int i = 0;

	if (!dirname) return NULL;

	dir = opendir (dirname);
	if (!dir) return NULL;

	while ((ent = readdir (dir))) {
		DList *item;

		/* Do not read '.' and '..' */
		i++;
		if (i <= 2) continue;

		item = calloc (sizeof (DList), 1);
		item->str = strdup (ent->d_name);
		item->next = NULL;
		if (list) {
			item->prev = list;
			list->next = item;
		} else {
			item->prev = NULL;
			liststart = item;
		}
		list = item;
	}
	closedir (dir);

	return liststart;
}

static void
d_list_free (DList *list)
{
	DList *l;

	if (!list) return;
	for (l = list; l; l = l->next)
		free (l->str);
	free (l);
}


static int
dirIsEmpty (const char *dirname)
{
	DList *list, *l;
	char *cwd;

	cwd = getcwd (NULL, 0);
	if (!cwd) return 0;
	if (chdir (dirname) != 0) {
		free (cwd);
		return 1;
	}


	list = d_list_new (".");
	if (!list) {
		chdir (cwd);
		free (cwd);
		return 1;
	}

	for (l = list; l; l = l->next) {
		struct stat buf;

		if (stat (l->str, &buf) != 0) break;
		if (S_ISDIR (buf.st_mode)) {
			if (!dirIsEmpty (l->str)) {
				chdir (cwd);
				free (cwd);
				return 0;
			}
		} else {
			chdir (cwd);
			free (cwd);
			return 0;
		}
	}
	d_list_free (list);


	chdir (cwd);
	free (cwd);
	return 1;
}


int
main (int argc, char *argv[])
{
	return !dirIsEmpty(argv[1]);
}
